from typing import Any

from django.contrib.gis.db.backends.base.operations import BaseSpatialOperations as BaseSpatialOperations
from django.db.backends.mysql.operations import DatabaseOperations as DatabaseOperations

class MySQLOperations(BaseSpatialOperations, DatabaseOperations):
    mysql: bool = ...
    name: str = ...
    geom_func_prefix: str = ...
    Adapter: Any = ...
    def select(self): ...
    def from_text(self): ...
    def gis_operators(self): ...
    disallowed_aggregates: Any = ...
    def unsupported_functions(self): ...
    def geo_db_type(self, f: Any): ...
    def get_distance(self, f: Any, value: Any, lookup_type: Any): ...
    def get_geometry_converter(self, expression: Any): ...
