from typing import Any, Optional

from django.contrib.gis.geos.geometry import GEOSGeometry as GEOSGeometry

class Point(GEOSGeometry):
    has_cs: bool = ...
    def __init__(
        self, x: Optional[Any] = ..., y: Optional[Any] = ..., z: Optional[Any] = ..., srid: Optional[Any] = ...
    ) -> None: ...
    def __iter__(self) -> Any: ...
    def __len__(self): ...
    @property
    def x(self): ...
    @x.setter
    def x(self, value: Any) -> None: ...
    @property
    def y(self): ...
    @y.setter
    def y(self, value: Any) -> None: ...
    @property
    def z(self): ...
    @z.setter
    def z(self, value: Any) -> None: ...
    @property
    def tuple(self): ...
    @tuple.setter
    def tuple(self, tup: Any) -> None: ...
    coords: Any = ...
