from typing import Any

GDAL_PIXEL_TYPES: Any
GDAL_INTEGER_TYPES: Any
GDAL_TO_CTYPES: Any
GDAL_RESAMPLE_ALGORITHMS: Any
GDAL_COLOR_TYPES: Any
VSI_FILESYSTEM_BASE_PATH: str
VSI_TAKE_BUFFER_OWNERSHIP: bool
VSI_DELETE_BUFFER_ON_READ: bool
