from typing import Any

from django.contrib.gis.gdal.base import GDALBase as GDALBase

class Layer(GDALBase):
    ptr: Any = ...
    def __init__(self, layer_ptr: Any, ds: Any) -> None: ...
    def __getitem__(self, index: Any): ...
    def __iter__(self) -> Any: ...
    def __len__(self): ...
    @property
    def extent(self): ...
    @property
    def name(self): ...
    @property
    def num_fields(self): ...
    @property
    def geom_type(self): ...
    @property
    def srs(self): ...
    @property
    def fields(self): ...
    @property
    def field_types(self): ...
    @property
    def field_widths(self): ...
    @property
    def field_precisions(self): ...
    spatial_filter: Any = ...
    def get_fields(self, field_name: Any): ...
    def get_geoms(self, geos: bool = ...): ...
    def test_capability(self, capability: Any): ...
