from ctypes import Structure
from typing import Any

class OGREnvelope(Structure): ...

class Envelope:
    def __init__(self, *args: Any) -> None: ...
    def __eq__(self, other: Any) -> Any: ...
    def expand_to_include(self, *args: Any): ...
    @property
    def min_x(self): ...
    @property
    def min_y(self): ...
    @property
    def max_x(self): ...
    @property
    def max_y(self): ...
    @property
    def ur(self): ...
    @property
    def ll(self): ...
    @property
    def tuple(self): ...
    @property
    def wkt(self): ...
