from typing import Any

from django.db.backends.postgresql.introspection import DatabaseIntrospection as DatabaseIntrospection

class PostGISIntrospection(DatabaseIntrospection):
    postgis_oid_lookup: Any = ...
    ignored_tables: Any = ...
    def get_field_type(self, data_type: Any, description: Any): ...
    def get_geometry_type(self, table_name: Any, description: Any): ...
