from typing import Any

from django.db.backends.base.schema import BaseDatabaseSchemaEditor as BaseDatabaseSchemaEditor

class DatabaseSchemaEditor(BaseDatabaseSchemaEditor):
    sql_rename_table: str = ...
    sql_alter_column_null: str = ...
    sql_alter_column_not_null: str = ...
    sql_alter_column_type: str = ...
    sql_delete_column: str = ...
    sql_delete_unique: str = ...
    sql_create_column_inline_fk: str = ...
    sql_delete_fk: str = ...
    sql_delete_index: str = ...
    sql_create_pk: str = ...
    sql_delete_pk: str = ...
    sql_create_index: str = ...
    @property
    def sql_delete_check(self): ...
    @property
    def sql_rename_column(self): ...
    def quote_value(self, value: Any): ...
    def skip_default(self, field: Any): ...
    def add_field(self, model: Any, field: Any) -> None: ...
