from typing import Any

from django.contrib.gis.geos.geometry import GEOSGeometry as GEOSGeometry

class Polygon(GEOSGeometry):
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...
    def __iter__(self) -> Any: ...
    def __len__(self): ...
    @classmethod
    def from_bbox(cls, bbox: Any): ...
    @property
    def num_interior_rings(self): ...
    exterior_ring: Any = ...
    shell: Any = ...
    @property
    def tuple(self): ...
    coords: Any = ...
    @property
    def kml(self): ...
