from typing import Any

from django.core.handlers import base as base
from django.http import HttpRequest as HttpRequest

logger: Any

class ASGIRequest(HttpRequest):
    body_receive_timeout: int = ...
    scope: Any = ...
    resolver_match: Any = ...
    script_name: Any = ...
    path_info: Any = ...
    path: Any = ...
    method: Any = ...
    META: Any = ...
    def __init__(self, scope: Any, body_file: Any) -> None: ...
    def GET(self): ...
    POST: Any = ...
    FILES: Any = ...
    def COOKIES(self): ...

class ASGIHandler(base.BaseHandler):
    request_class: Any = ...
    chunk_size: Any = ...
    def __init__(self) -> None: ...
    async def __call__(self, scope: Any, receive: Any, send: Any) -> None: ...
    async def read_body(self, receive: Any): ...
    def create_request(self, scope: Any, body_file: Any): ...
    def handle_uncaught_exception(self, request: Any, resolver: Any, exc_info: Any): ...
    async def send_response(self, response: Any, send: Any) -> None: ...
    @classmethod
    def chunk_bytes(cls, data: Any) -> None: ...
    def get_script_prefix(self, scope: Any): ...
